
(* ::Section:: *)
(* Public Declaration Part: executes in Theorema`Provers` *)

With[ {lang = "English"},

MessageName[ reductionRingProver, "usage", lang] = "Inference rules for the theory of reduction rings";

MessageName[ RRPorderingGoal, "usage", lang] = "Prove goal in the theory of orderings";
MessageName[ RRPorderingKB, "usage", lang] = "Handle knowledge-base contradictions in the theory of orderings";
MessageName[ RRPorderingEqualGoal, "usage", lang] = "Prove equality in the theory of orderings";

MessageName[ RRPmembershipDomainTuples, "usage", lang] = "Prove membership in domain-tuples";
MessageName[ RRPmembershipDomainSets, "usage", lang] = "Prove membership in domain-sets";

MessageName[ RRPmembershipCommRings1, "usage", lang] = "Prove membership in commutative rings with unit";
MessageName[ RRPcommRing1Equal, "usage", lang] = "Prove equality in commutative rings with unit";
MessageName[ RRPcommRing1EqualKB, "usage", lang] = "Simplify known equalities in commutative rings with unit";

MessageName[ RRPmembershipCCMonoids, "usage", lang] = "Prove membership in cancellative commutative monoids";
MessageName[ RRPCCMonoidEqual, "usage", lang] = "Prove equality in cancellative commutative monoids";

MessageName[ RRPmembershipIntegerInterval, "usage", lang] = "Prove membership in integer intervals";

MessageName[ RRPsplitConjunction, "usage", lang] = "Split conjunction in the goal into two parts";
MessageName[ RRPcheckConditions, "usage", lang] = "Check conditions of rewrite rules";

] (* With *)


(* ::Section:: *)
(* Private Implementation Part: executes in Theorema`Provers`Private` *)

Begin["`Private`"]

With[ {lang = "English"},

translate[ "CommRing1 Rules", lang] = "Commutative Rings with Unit";
translate[ "Domain-Collection Rules", lang] = "Domain Collections";
translate[ "Integer Rules", lang] = "Integers";
translate[ "Interactive Rules", lang] = "Interactive Rules";
translate[ "Miscellaneous Rules", lang] = "Miscellaneous";
translate[ "CCMonoid Rules", lang] = "Cancellative Commutative Monoids";
translate[ "Ordering Rules", lang] = "Order Relations";
translate[ "Prover for the theory of Reduction Rings", lang] = "Inference Rules for the Theory of Reduction Rings";
translate[ "Propositional Logic Rules", lang] = "Propositional Logic";
translate[ "Quantifier Rules", lang] = "Logical Quantifiers";
translate[ "Rules for Rewriting", lang] = "Rewriting";


intToText[ lang, a_Integer] :=
	Switch[ a,
		2, "two",
		3, "three",
		4, "four",
		5, "five",
		6, "six",
		7, "seven",
		8, "eight",
		9, "nine",
		10, "ten",
		11, "eleven",
		12, "twelve",
		_, ToString[ a]
	];


(* ::Section:: *)
(* Ordering *)

proofStepText[ RRPorderingGoal, lang, {{g_, o_}}, {}, ___, "data" -> {u_List, r:("irreflNot"|"refl")}, ___] :=
	{textCell[ "Our current goal ", formulaReference[ g],
		" holds in the theory of " <> orderTypeToString[ r] <> " order relations; This follows readily from ",
		formulaReferenceSequence[ Prepend[ u, o], lang], "."]};
proofStepText[ RRPorderingGoal, lang, {{g_, o_}}, {{cond_}}, ___, "data" -> {elem_, dom_, r:("irreflNot"|"refl")}, ___] :=
	{textCell[ "Our current goal ", formulaReference[ g],
		" holds in the theory of " <> orderTypeToString[ r] <> " order relations (due to ", formulaReference[ o],
		"), if ", inlineTheoremaExpression[ elem], " is an element of domain ",
		inlineTheoremaExpression[ dom]],
	textCell[ "Hence, all we have to show is"], goalCell[ cond, "."]};
	
proofStepText[ RRPorderingGoal, lang, {{g_, o_}}, {}, ___, "data" -> {_, rel_, _, _, elems_, t_List, d_List, "transAsymNot"}, ___] :=
	With[ {text = If[ elems === {}, "", " by transitivity"]},
		{textCell[ "Our current goal ", formulaReference[ g], " follows from ", formulaReferenceSequence[ Join[ t, d], lang], text <> ", using the fact that ",
			inlineTheoremaExpression[ rel], " is an asymmetric order relation (due to ", formulaReference[ o], ")."]}
	];
proofStepText[ RRPorderingGoal, lang, {{g_, o_}}, {{newG_, ___}}, ___, "data" -> {_, rel_, _, _, elems_, t_List, d_List, type:("transAsymNot"|"transAntisymNot")}, ___] :=
	With[ {text = If[ elems === {}, "", "by transitivity, "], tp = If[ type === "transAsymNot", "asymmetric", "antisymmetric"]},
		{textCell[ "Our current goal ", formulaReference[ g], " can be reduced to"], goalCell[ newG, ""],
		textCell[ text <> "using ", formulaReferenceSequence[ Join[ t, d], lang], " and the fact that ",
			inlineTheoremaExpression[ rel], " is an " <> tp <> " order relation (due to ", formulaReference[ o], ")."]}
	];
	
subProofHeader[ RRPorderingGoal, lang, _, {{_, cond_}}, ___, "data" -> {__, "transAntisymNot"}, ___, {1}] :=
	{textCell[ "Before, however, we need to make sure that also"], goalCell[ cond, ""], textCell[ "holds."]};
subProofHeader[ RRPorderingGoal, lang, _, {{newG_, cond_}}, ___, "data" -> {__, "transAntisymNot"}, ___, {2}] :=
	{textCell[ "Now we proceed with proving ", formulaReference[ newG], ", additionally knowing"], assumptionCell[ cond, "."]};
	
proofStepText[ RRPorderingGoal, lang, {{g_, _}}, {}, ___, "data" -> {_, _, _, _, {}, t_List, _List, "trans"}, ___] :=
	{textCell[ "Our current goal ", formulaReference[ g], " follows readily from ", formulaReferenceSequence[ t, lang], "."]};
proofStepText[ RRPorderingGoal, lang, {{g_, o_}}, {}, ___, "data" -> {_, rel_, _, _, _, t_List, d_List, "trans"}, ___] :=
	{textCell[ "Our current goal ", formulaReference[ g], " follows from ", formulaReferenceSequence[ Join[ t, d], lang], " by transitivity, using the fact that ",
		inlineTheoremaExpression[ rel], " is an order relation (due to ", formulaReference[ o], ")."]};
proofStepText[ RRPorderingGoal, lang, {{g_, o_}}, {{newG_}}, ___, "data" -> {_, rel_, _, _, _, t_List, d_List, "trans"}, ___] :=
	{textCell[ "Our current goal ", formulaReference[ g], " can be reduced to"], goalCell[ newG, ""],
	textCell[ "by transitivity, using ", formulaReferenceSequence[ Join[ t, d], lang], " and the fact that ",
		inlineTheoremaExpression[ rel], " is an order relation (due to ", formulaReference[ o], ")."]};
	
proofStepText[ RRPorderingGoal, lang, {{g_, o_}}, {{newG_}}, ___, "data" -> {u_List, "orderEmbedding"}, ___] :=
	{textCell[ "Our current goal ", formulaReference[ g],
		" can be reduced to"], goalCell[ newG, ""],
	textCell[ "because of ", formulaReference[ o], " (and domain membership of the terms involved follows readily from ",
		formulaReferenceSequence[ u, lang], ")."]};
proofStepText[ RRPorderingGoal, lang, {{g_, o_}}, {{newG_, cond_}}, ___, "data" -> {dom_, u_List, "orderEmbedding"}, ___] :=
	Module[ {text},
		text = {textCell[ "Our current goal ", formulaReference[ g],
			" can be reduced to "], goalCell[ newG, ""], textCell[ "due to ", formulaReference[ o],
			", under the condition that all terms involved belong to the domain ", inlineTheoremaExpression[ dom], "."]};
		If[ u =!= {},
			AppendTo[ text, textCell[ "For some terms domain membership follows readily from ", formulaReferenceSequence[ u, lang], "."]]
		];
		text
	];
subProofHeader[ RRPorderingGoal, lang, _, {{_, cond_}}, ___, "data" -> {__, "orderEmbedding"}, ___, {1}] :=
	{textCell[ "Hence, we need to make sure that"], goalCell[ cond, ""], textCell[ "holds."]};
subProofHeader[ RRPorderingGoal, lang, _, {{newG_, cond_}}, ___, "data" -> {__, "orderEmbedding"}, ___, {2}] :=
	{textCell[ "Now we proceed with proving ", formulaReference[ newG], ", additionally knowing"], assumptionCell[ cond, "."]};
		
proofStepText[ RRPorderingKB, lang, {{f_, o_}}, {}, ___, "data" -> {u_List, rule_}, ___] :=
	{textCell[ "Formula ", formulaReference[ f],
		" in our assumptions is a contradiction in the theory of " <> orderTypeToString[ rule] <> " order relations; This follows readily from ",
		formulaReferenceSequence[ Prepend[ u, o], lang], "."]};
proofStepText[ RRPorderingKB, lang, {{f_, o_}}, {{cond_}}, ___, "data" -> rule_, ___] :=
	{textCell[ "Formula ", formulaReference[ f],
		" in our assumptions seems to be a contradiction in the theory of " <> orderTypeToString[ rule] <> " order relations (due to ",
		formulaReference[ o], ")."],
	textCell[ "Hence, all we have to show is"], goalCell[ cond, "."]};
	
proofStepText[ RRPorderingEqualGoal, lang, {{g_, o_, d_}}, {{newG_}}, ___, "data" -> {u_List, dom_, t_, rel_}, ___] :=
	{textCell[ "For proving the equality ", formulaReference[ g],
		" we use the fact that the two sides of the equality are elements of domain ",
		inlineTheoremaExpression[ dom], ", which is ordered by the " <> orderTypeToString[ t] <> " ordering ",
		inlineTheoremaExpression[ rel], " (according to ", formulaReference[ o], "); Domain membership follows readily from ",
		formulaReferenceSequence[ Prepend[ u, d], lang], "."],
	textCell[ "Therefore, we now prove the equivalent goal"], goalCell[ newG, "."]};
proofStepText[ RRPorderingEqualGoal, lang, {{g_, o_, d_}}, {{newG_, cond_}}, ___, "data" -> {dom_, t_, rel_}, ___] :=
	{textCell[ "For proving the equality ", formulaReference[ g],
		" we use the fact that at least one side of the equality is an element of domain ",
		inlineTheoremaExpression[ dom], ", which is ordered by the " <> orderTypeToString[ t] <> " ordering ",
		inlineTheoremaExpression[ rel], " (according to ", formulaReference[ o], "); Domain membership follows readily from ",
		formulaReference[ d], "."]};
subProofHeader[ RRPorderingEqualGoal, lang, _, {{_, cond_FML$}}, ___, "data" -> {dom_, __}, ___, {1}] :=
	{textCell[ "Therefore, we first make sure that the other side of the equality is also an element of ",
		inlineTheoremaExpression[ dom], ", i.e. we prove"], goalCell[ cond, "."]};
subProofHeader[ RRPorderingEqualGoal, lang, _, {{newG_FML$, cond_FML$}}, ___, {2}] :=
	{textCell[ "Now we proceed with proving the new goal"], goalCell[ newG, ""],
		textCell[ "under the additional assumption"], assumptionCell[ cond, "."]};
		
orderTypeToString[ Theorema`Knowledge`isPartIrreflOrder$TM] = "partial irreflexive";
orderTypeToString[ Theorema`Knowledge`isPartReflOrder$TM] = "partial reflexive";
orderTypeToString[ Theorema`Knowledge`isTotalIrreflOrder$TM] = "total irreflexive";
orderTypeToString[ "refl"|"reflNot"] = "reflexive";
orderTypeToString[ "irrefl"|"irreflNot"] = "irreflexive";
orderTypeToString[ _] = "unknown";
	

(* ::Section:: *)
(* Domain-Collections *)

proofStepText[ n:(RRPmembershipDomainTuples|RRPmembershipDomainSets), lang, {{g_}, {}}, {}, ___] :=
	{textCell[ "The domain-membership goal ", formulaReference[ g], " holds in general in the theory of " <> theoryText[ n] <> "s."]};
proofStepText[ n:(RRPmembershipDomainTuples|RRPmembershipDomainSets), lang, {{g_}, refs_List}, {}, ___] :=
	{textCell[ "The domain-membership goal ", formulaReference[ g], " follows from ",
		formulaReferenceSequence[ refs, lang], " in the theory of " <> theoryText[ n] <> "s."]};
proofStepText[ n:(RRPmembershipDomainTuples|RRPmembershipDomainSets), lang, {{g_}, refs_List}, {}, ___, "domMembership" -> False, ___] :=
	{textCell[ "The goal ", formulaReference[ g], " follows from ",
		formulaReferenceSequence[ refs, lang], " in the theory of " <> theoryText[ n] <> "s."]};
proofStepText[ n:(RRPmembershipDomainTuples|RRPmembershipDomainSets), lang, {{g_}, {}}, {{newG_}}, ___] :=
	{textCell[ "The domain-membership goal ", formulaReference[ g], " can be reduced to"],
	goalCell[ newG, ""],
	textCell["in the theory of " <> theoryText[ n] <> "s."]};
proofStepText[ n:(RRPmembershipDomainTuples|RRPmembershipDomainSets), lang, {{g_}, refs_List}, {{newG_}}, ___] :=
	{textCell[ "The domain-membership goal ", formulaReference[ g], " can be reduced to"],
	goalCell[ newG, ""],
	textCell[ "using ", formulaReferenceSequence[ refs, lang], "."]};
	

(* ::Section:: *)
(* CCMonoid and CommRing1 *)
	
proofStepText[ n:(RRPmembershipCommRings1|RRPcommRing1Equal|RRPmembershipCCMonoids|RRPCCMonoidEqual), lang, {{g_, df_}, {}}, {}, ___, "dom" -> dom_, ___] :=
	{textCell[ "Since ", domainExpression[ dom], " is a " <> theoryText[ n, dom] <> " due to ", formulaReference[ df],
		", the " <> goalKind[ n, False], formulaReference[ g], " clearly holds."]};
proofStepText[ n:(RRPmembershipCommRings1|RRPcommRing1Equal|RRPmembershipCCMonoids|RRPCCMonoidEqual), lang, {{g_, df_}, refs_List}, {}, ___, "dom" -> dom_, ___] :=
	{textCell[ "Since ", domainExpression[ dom], " is a " <> theoryText[ n, dom] <> " due to ", formulaReference[ df],
		", the " <> goalKind[ n, False], formulaReference[ g], " follows from ",
		formulaReferenceSequence[ refs, lang], "."]};
proofStepText[ n:(RRPmembershipCommRings1|RRPcommRing1Equal|RRPmembershipCCMonoids|RRPCCMonoidEqual), lang, {{g_, df_}, {}}, {newG_List}, ___, "dom" -> dom_, ___] :=
	{textCell[ "Since ", domainExpression[ dom], " is a " <> theoryText[ n, dom] <> " due to ", formulaReference[ df],
		", the " <> goalKind[ n, True], formulaReference[ g], " can be reduced to" <> conjunctionText[ newG]],
	goalListCells[ newG, ",", "."]};
proofStepText[ n:(RRPmembershipCommRings1|RRPcommRing1Equal|RRPmembershipCCMonoids|RRPCCMonoidEqual), lang, {{g_, df_}, refs_List}, {newG_List}, ___, "dom" -> dom_, ___] :=
	{textCell[ "Since ", domainExpression[ dom], " is a " <> theoryText[ n, dom] <> " due to ", formulaReference[ df],
		", the " <> goalKind[ n, True], formulaReference[ g], " can be reduced to" <> conjunctionText[ newG]],
	goalListCells[ newG, ",", ""],
	textCell[ "using ", formulaReferenceSequence[ refs, lang], "."]};
	
proofStepText[ n:(RRPCCMonoidEqualKB|RRPcommRing1EqualKB), lang, {u_List, {}, {dref_}}, {gen_List}, ___, "dom" -> dom_, ___] :=
	{textCell[ "Since ", domainExpression[ dom], " is a " <> theoryText[ n, dom] <> " due to ", formulaReference[ dref],
		", the assumption" <> If[ Length[ u] === 1, " ", "s "], formulaReferenceSequence[ u, lang], " can trivially be simplified to"],
		assumptionListCells[ gen, ",", "."]};
proofStepText[ n:(RRPCCMonoidEqualKB|RRPcommRing1EqualKB), lang, {u_List, refs_List, {dref_}}, {gen_List}, ___, "dom" -> dom_, ___] :=
	{textCell[ "Since ", domainExpression[ dom], " is a " <> theoryText[ n, dom] <> " due to ", formulaReference[ dref],
		", the assumption" <> If[ Length[ u] === 1, " ", "s "], formulaReferenceSequence[ u, lang], " can be simplified to"],
		assumptionListCells[ gen, ",", ""], textCell[ "because of ", formulaReferenceSequence[ refs, lang], "."]};
proofStepText[ n:(RRPCCMonoidEqualKB|RRPcommRing1EqualKB), lang, {u_List, {}, {dref_}}, {gen_List, conds_List}, ___, "dom" -> dom_, ___] :=
	{textCell[ "Since ", domainExpression[ dom], " is a " <> theoryText[ n, dom] <> " due to ", formulaReference[ dref],
		", the assumption" <> If[ Length[ u] === 1, " ", "s "], formulaReferenceSequence[ u, lang], " can be simplified to"],
		assumptionListCells[ gen, ",", ""],
		textCell[ "if" <> conjunctionText[ conds]],
		goalListCells[ conds, ",", ""], textCell[ "can be shown."]};
proofStepText[ n:(RRPCCMonoidEqualKB|RRPcommRing1EqualKB), lang, {u_List, refs_List, {dref_}}, {gen_List, conds_List}, ___, "dom" -> dom_, ___] :=
	{textCell[ "Since ", domainExpression[ dom], " is a " <> theoryText[ n, dom] <> " due to ", formulaReference[ dref],
		", the assumption" <> If[ Length[ u] === 1, " ", "s "], formulaReferenceSequence[ u, lang], " can be simplified to"],
		assumptionListCells[ gen, ",", ""],
		textCell[ "because of ", formulaReferenceSequence[ refs, lang], ", if in addition" <> conjunctionText[ conds]],
		goalListCells[ conds, ",", ""], textCell[ "can be shown."]};
	
subProofHeader[ RRPmembershipCCMonoids|RRPmembershipCommRings1|RRPCCMonoidEqual|RRPcommRing1Equal|RRPCCMonoidEqualKB|RRPcommRing1EqualKB, lang, rest___] :=
	subProofHeader[ Theorema`Provers`RRPandGoal, lang, rest];
subProofHeader[ RRPCCMonoidEqualKB|RRPcommRing1EqualKB, lang, u_, {_List, conds_List}, rest___, {p_}] /; p <= Length[ conds] :=
	subProofHeader[ RRPandGoal, lang, u, {conds}, rest, {p}];
subProofHeader[ RRPCCMonoidEqualKB|RRPcommRing1EqualKB, lang, _, {gen_List, _List}, ___, {_}] :=
	{textCell[ "We continue proving our original goal, additionally knowing ", formulaReferenceSequence[ gen, lang], "."]};
	
goalKind[ RRPmembershipCCMonoids|RRPmembershipCommRings1, False] = "domain-membership goal ";
goalKind[ RRPmembershipCCMonoids|RRPmembershipCommRings1, True] = "goal ";
goalKind[ RRPCCMonoidEqual|RRPcommRing1Equal, _] = "equality ";
	
domainExpression[ {dom_}] := inlineTheoremaExpression[ dom];
domainExpression[ dom:{_, __}] := Cell[ FormBox[ RowBox[ {"(", RowBox[ Riffle[ Map[ theoremaBoxes, dom], ","]], ")"}], TheoremaForm]];
domainExpression[ dom_] := inlineTheoremaExpression[ dom];

theoryText[ RRPmembershipDomainTuples] = "domain-tuple";
theoryText[ RRPmembershipDomainSets] = "domain-set";
theoryText[ RRPmembershipCCMonoids|RRPCCMonoidEqual|RRPCCMonoidEqualKB] = "multiplicative cancellative commutative monoid";
theoryText[ RRPmembershipCCMonoids|RRPCCMonoidEqual|RRPCCMonoidEqualKB, {_, _, _}] = "cancellative commutative monoid";
theoryText[ RRPmembershipCommRings1|RRPcommRing1Equal|RRPcommRing1EqualKB] = "commutative ring with unit";
theoryText[ t_, _] = theoryText[ t];
theoryText[ _] = "unknown structure";

conjunctionText[ {_, __}] = " the conjunction of";
conjunctionText[ _] = "";

proofStepText[ RRPsplitConjunction, lang, {{g_}}, {l_List}, ___] :=
	{textCell[ "We split the conjunction ", formulaReference[ g], " into the ", intToText[ lang, Length[ l]], " parts"],
		goalListCells[ l, ",", ""],
		textCell[ "and prove the individual parts separately."]};
subProofHeader[ RRPsplitConjunction, lang, rest___] :=
	subProofHeader[ RRPandGoal, lang, rest];

goalListCells[ {f_}, _String, punct_String] :=
	goalCell[ f, punct];
goalListCells[ {f__, l_}, sep_String, punct_String] :=
	Module[{initial, term},
		initial = Map[ goalCell[ #, sep]&, {f}];
		term = goalCell[ l, punct];
		cellGroup[ Append[ initial, term]]
	];
	
	
(* ::Section:: *)
(* Integer Intervals *)
	
proofStepText[ RRPmembershipIntegerInterval, lang, {{g_}, {}}, {}, ___] :=
	{textCell[ "The goal ", formulaReference[ g], " clearly holds in the theory of integer intervals."]};
proofStepText[ RRPmembershipIntegerInterval, lang, {{g_}, refs_List}, {}, ___] :=
	{textCell[ "The goal ", formulaReference[ g], " follows readily from ", formulaReferenceSequence[ refs, lang], " in the theory of integer intervals."]};
proofStepText[ RRPmembershipIntegerInterval, lang, {{g_}, {}}, {{newG_}}, ___] :=
	{textCell[ "The goal ", formulaReference[ g], " can be reduced to"],
	goalCell[ newG, ""],
	textCell[ "in the theory of integer intervals."]};
proofStepText[ RRPmembershipIntegerInterval, lang, {{g_}, refs_List}, {{newG_}}, ___] :=
	{textCell[ "The goal ", formulaReference[ g], " can be reduced to"],
	goalCell[ newG, ""],
	textCell[ "in the theory of integer intervals, using ", formulaReferenceSequence[ refs, lang], "."]};
	
] (* With *)

End[]